#include "ReleaseHandler.hxx"

namespace SALTO
{
	void ReleaseHandlerConfig::DefaultInit()
	{
		AddAll();
		UpdateData();
	}

	ReleaseHandler::ReleaseHandler():
		mUseRandomLoop( false ), 
		mUsePitchCorrection( true ), mRandomRange( 8 ), mFrameCounterRelease( 1 ), mNumFramesRelease( 6 ),
		mInUseRandomLoop( "Use Random Loop In Control", this, &ReleaseHandler::UseRandomLoopCB ),
		mInRandomRange( "Random Range In Control", this ),
		mInUsePicthCorrection( "Use Pitch Correction In Control", this ),
		mOut_InLoopSynthesis( "In Loop Synthesis Out Control", this ),
		mOutUseRandomLoop( "Use Random Loop Out Control", this ),
		mOutPitchFactor( "Pitch Factor Out Control", this ),
		mOutRandomRange( "Random Range Out Control", this ),
		mOutTargetFreq( "Target Frequency Out Control", this ),
		mOutMagInterpolFactor( "Magnitude Interpolation Factor Out Control", this ),
		mOutMagGain( "Magnitude Gain Out Control", this )
	{
		ReleaseHandlerConfig cfg;

		Configure( cfg );
	}

	ReleaseHandler::ReleaseHandler( ReleaseHandlerConfig &cfg ):
		mUseRandomLoop( false ), 
		mUsePitchCorrection( true ), mRandomRange( 8 ),
		mFrameCounterRelease( 1 ), mNumFramesRelease( 6 ),
		mInUseRandomLoop( "Use Random Loop In Control", this, &ReleaseHandler::UseRandomLoopCB ),
		mInRandomRange( "Random Range In Control", this ),
		mInUsePicthCorrection( "Use Pitch Correction In Control", this ),
		mOut_InLoopSynthesis( "In Loop Synthesis Out Control", this ),
		mOutUseRandomLoop( "Use Random Loop Out Control", this ),
		mOutPitchFactor( "Pitch Factor Out Control", this ),
		mOutRandomRange( "Random Range Out Control", this ),
		mOutTargetFreq( "Target Frequency Out Control", this ),
		mOutMagInterpolFactor( "Magnitude Interpolation Factor Out Control", this ),
		mOutMagGain( "Magnitude Gain Out Control", this )
	{
		Configure( cfg );
	}

	ReleaseHandler::~ReleaseHandler()
	{}

	bool ReleaseHandler::ConcreteConfigure( const ProcessingConfig &c )
	{
		CopyAsConcreteConfig(mConfig, c);

		return true;
	}

	bool ReleaseHandler::ConcreteStart()
	{
		return true;
	}

	bool ReleaseHandler::ConcreteStop()
	{
		return true;
	}

	int ReleaseHandler::UseRandomLoopCB( TControlData value )
	{
		if( value > 0 )		mUseRandomLoop = true;
		else				mUseRandomLoop = false;

		return 0;
	}

	int ReleaseHandler::UsePitchCorrectionCB( TControlData value )
	{
		if( value > 0 )		mUsePitchCorrection = true;
		else				mUsePitchCorrection = false;

		return 0;
	}

	int ReleaseHandler::RandomRangeCB( TControlData value )
	{
		mRandomRange = value;

		return 0;
	}

	bool ReleaseHandler::Do()
	{
		return true;
	}

	void ReleaseHandler::DoInterpolatingReleaseSynthesis( CSaltoSynthFrame &pSynthFrame,
														  SynthesisState   &synthState,
														  double gain )
	{
		pSynthFrame.ClearSpectrum();

		mOut_InLoopSynthesis.SendControlAsBoolean( synthState.GetInLoopSynthesis() );
		mOutUseRandomLoop.SendControlAsBoolean( mUseRandomLoop );
		mOutRandomRange.SendControl( mRandomRange );
		mOutTargetFreq.SendControl( synthState.GetCurrentStatTemplateFundFreq() );
		mOutMagInterpolFactor.SendControl( synthState.GetIPFactor() );
		mOutMagGain.SendControl( synthState.GetIndividualGain() / ( mFrameCounterRelease * mFrameCounterRelease ) );

	/*	mpInterpolPO.DoMagInterpolation( *mpCurrSpectralFrameBase,
										  *mpCurrPeakArrayTarget,
	//									  mpParams->GetCurrentStatTemplateFundFreq(),
	//									  mIPFactor,
	//									  mIndividualGain / ( currRelFrame * currRelFrame ),
										  *mpSynthFrame );
*/
		double pitchCorrectionFactor=1;
		
		if ( mUsePitchCorrection && pSynthFrame.GetFundFreq()!=0)
			pitchCorrectionFactor= synthState.GetPitchReference() / pSynthFrame.GetFundFreq();

		mOutPitchFactor.SendControl( synthState.GetPitchModFactor() * pitchCorrectionFactor );
//		mpInterpolPO.DoPitchMod( *pSynthFrame);
	}

	void ReleaseHandler::InitReleaseSynthesis( SynthesisState &synthState )
	{
		mNumFramesRelease = 6;
		mFrameCounterRelease = 1;
		synthState.SetTransitionMode( false );
	}

	bool ReleaseHandler::Do( CSaltoSynthFrame &pSynthFrame,
							 SynthesisState   &synthState,
							 double gain )
	{
		DoInterpolatingReleaseSynthesis( pSynthFrame, synthState, gain );

//		DoReleaseSynthesisProcess( synthState );
	
		mFrameCounterRelease++;
	
/*		if ( mFrameCounterRelease >= mNumFramesRelease )
			mOutState.SendControl( 0 );
		//	mState = Idle; 
*/
		return true;
	}

}
