#include "StationaryHandler.hxx"

namespace SALTO
{
	void StationaryHandlerConfig::DefaultInit( void )
	{
		AddAll();
		UpdateData();
	}

	StationaryHandler::StationaryHandler():
		mUseRandomLoop( false ), mInterpolFactor( 0.0 ),mUseRandomDeviations( false ),
		mRandomRange( 8.0 ), mUsePitchCorrection( true ),
		mInUseRandomLoop( "Random Loop In Control", this, &StationaryHandler::UseRandomLoopCB ),
		mInInterpolFactor( "Interpolation Factor In Control", this, &StationaryHandler::InterpolFactorCB ),
		mInUseRandomDeviations( "Use Random Deviations In Control", this, &StationaryHandler::UseRandomDeviationsCB ),
		mInRandomRange( "Random Range In Control", this, &StationaryHandler::RandomRangeCB ),
		mInUsePitchCorrection( "Use Pitch Correction In Control", this, &StationaryHandler::UsePitchCorrectionCB ),
		mOutAttackTimbreLevel( "Attack Timbre Level Out Control", this ),
		mOutLastAlignedFrame( "Last Aligned Frame Out Control", this ),
		mOutResGain( "Residual Gain Out Control", this ),
		mOutResFreq( "Residual Frequency Out Control", this ),
		mOut_InLoopSynthesis( "In Loop Synthesis Out Control", this ),
		mOutUseRandomLoop( "Use Random Loop Out Control", this ),
		mOutUseRandomDeviations( "Use Random Deviations Out Control", this ),
		mOutPitchFactor( "Pitch Factor Out Control", this ),
		mOutRandomRange( "Random Range Out Control", this ),
		mOutTargetFreq( "Target Frequency Out Control", this ),
		mOutMagInterpolFactor( "Magnitude Interpolation Factor Out Control", this ),
		mOutMagGain( "Magintude Gain Out Control", this ),
		mOutFreqInterpolFactor( "Frequency Interpolation Factor Out Control", this ),
		mOutMagInterpolFactor2( "Magnitude  Interpolation Factor 2 Out Control", this ),
		mOutAttackResGain( "Attack Residual Gain Out Control", this )
	{
		StationaryHandlerConfig cfg;

		Configure( cfg );
	}

	StationaryHandler::StationaryHandler( StationaryHandlerConfig& cfg ):
		mUseRandomLoop( false ), mInterpolFactor( 0.0 ),mUseRandomDeviations( false ),
		mRandomRange( 8.0 ), mUsePitchCorrection( true ),
		mInUseRandomLoop( "Random Loop In Control", this, &StationaryHandler::UseRandomLoopCB ),
		mInInterpolFactor( "Interpolation Factor In Control", this, &StationaryHandler::InterpolFactorCB ),
		mInUseRandomDeviations( "Use Random Deviations In Control", this, &StationaryHandler::UseRandomDeviationsCB ),
		mInRandomRange( "Random Range In Control", this, &StationaryHandler::RandomRangeCB ),
		mInUsePitchCorrection( "Use Pitch Correction In Control", this, &StationaryHandler::UsePitchCorrectionCB ),
		mOutAttackTimbreLevel( "Attack Timbre Level Out Control", this ),
		mOutLastAlignedFrame( "Last Aligned Frame Out Control", this ),
		mOutResGain( "Residual Gain Out Control", this ),
		mOutResFreq( "Residual Frequency Out Control", this ),
		mOut_InLoopSynthesis( "In Loop Synthesis Out Control", this ),
		mOutUseRandomLoop( "Use Random Loop Out Control", this ),
		mOutUseRandomDeviations( "Use Random Deviations Out Control", this ),
		mOutPitchFactor( "Pitch Factor Out Control", this ),
		mOutRandomRange( "Random Range Out Control", this ),
		mOutTargetFreq( "Target Frequency Out Control", this ),
		mOutMagInterpolFactor( "Magnitude Interpolation Factor Out Control", this ),
		mOutMagGain( "Magintude Gain Out Control", this ),
		mOutFreqInterpolFactor( "Frequency Interpolation Factor Out Control", this ),
		mOutMagInterpolFactor2( "Magnitude  Interpolation Factor 2 Out Control", this ),
		mOutAttackResGain( "Attack Residual Gain Out Control", this )
	{
		Configure( cfg );
	}

	StationaryHandler::~StationaryHandler()
	{
	}

	bool StationaryHandler::ConcretStart()
	{
		return true;
	}

	bool StationaryHandler::ConcretStop()
	{
		return true;
	}

	bool StationaryHandler::ConcreteConfigure( const ProcessingConfig& c )
	{
		CopyAsConcreteConfig(mConfig, c);

		return true;
	}

	int StationaryHandler::UseRandomLoopCB( TControlData value )
	{
		if( value > 0 )		mUseRandomLoop = true;
		else				mUseRandomLoop = false;

		return 0;
	}

	int StationaryHandler::UseRandomDeviationsCB( TControlData value )
	{
		if( value > 0 )		mUseRandomDeviations = true;
		else				mUseRandomDeviations = false;

		return 0;
	}

	int StationaryHandler::UsePitchCorrectionCB( TControlData value )
	{
		if( value > 0 )		mUsePitchCorrection = true;
		else				mUsePitchCorrection = false;

		return 0;
	}

	int StationaryHandler::InterpolFactorCB( TControlData value )
	{
		mInterpolFactor = value;

		return 0;
	}

	int StationaryHandler::RandomRangeCB( TControlData value )
	{
		mRandomRange = value;

		return 0;
	}

	bool StationaryHandler::Do()
	{
		return true;
	}

	bool StationaryHandler::Do( CSaltoSynthFrame  &pSynthFrame,
						   Frame			 *pCurrSpectralFrameBase,										
						   SynthesisState    &synthState,
						   Frame			 *pCurrSpectralFrameResidual,
						   SpectralPeakArray *pCurrPeakArrayTarget)
	{

		DoInterpolatingSynthesis( pSynthFrame, pCurrSpectralFrameBase, 
								  synthState, pCurrSpectralFrameResidual, pCurrPeakArrayTarget );

//		DoStationarySynthesisProcess( synthState );

		return true;
	}

	void StationaryHandler::DoInterpolatingSynthesis( CSaltoSynthFrame  &pSynthFrame,
													  Frame			    *pCurrSpectralFrameBase,										
													  SynthesisState    &synthState,
													  Frame			    *pCurrSpectralFrameResidual,
													  SpectralPeakArray *pCurrPeakArrayTarget)
	{
		CSaltoDataManagment* handleDB = CSaltoDataManagment::GetSaltoDBHandle();
		
		pSynthFrame.ClearSpectrum();  // reset synth frame OPTIMIZE
		
		// -- data managment BaseFrame --
		if ( mUseRandomLoop )
			{
				if ( synthState.GetFrameCounterBase() == synthState.GetLoopStart() )
					{
						synthState.SetLoopDirectionFW(true);
						synthState.SetInLoopSynthesis(true);       // indicates that we are in loop mode
						int randRange = synthState.GetLimitLoopEnd() - synthState.GetLoopStart() ;
						//mLoopEnd = (Random()+32767)/65535.0*randRange+mLoopStart;	
						synthState.SetLoopEnd( ((TFloat32)rand()/RAND_MAX)*
												randRange+synthState.GetLoopStart() );	
					}
				else if (synthState.GetFrameCounterBase() > synthState.GetLoopEnd() )
					{
						synthState.SetLoopDirectionFW(false);
						synthState.SetInLoopSynthesis(true);       // indicates that we are in loop mode
						int randRange = synthState.GetLoopEnd() - synthState.GetLimitLoopStart();
						synthState.SetLoopStart( ((TFloat32)rand()/RAND_MAX)*
												randRange+synthState.GetLimitLoopStart() );
					}
			}
		else  // repeat always one frame...
			{
				if (synthState.GetFrameCounterBase() >= synthState.GetLoopStart() )    
					{
						synthState.SetFrameCounterBase( synthState.GetLoopStart() );
						synthState.SetInLoopSynthesis(true);
					}
			}	
		
		pCurrSpectralFrameBase = handleDB->GetSpectralFrame( synthState.GetSegPositionBase() ,
															 synthState.GetFrameCounterBase() );
		
		// increase or decrease position of readout frame depending on loop direction
		if ( synthState.GetLoopDirectionFW() )
			synthState.SetFrameCounterBase( synthState.GetFrameCounterBase() + 1 );
		else
			synthState.SetFrameCounterBase( synthState.GetFrameCounterBase() - 1 );
		
		if(synthState.GetFrameCounterBase() >= synthState.GetNumFramesBase() )// security-stop at end of the segment
			{
				synthState.SetFrameCounterBase( 0 );
				//mpParams->SetPlay(false);
				return;
			}
		
		// -- data managment TargetFrame --
		pCurrPeakArrayTarget = synthState.GetCurrentStatTemplatePtr();
		
		// -- data managment attack residual --
		TIndex nResFrames= handleDB->GetFramesInSeg(synthState.GetAttackTimbre().GetPosition());
		
		if ( synthState.GetFrameCounterAttackResidual() >= nResFrames)
			synthState.SetFrameCounterAttackResidual( nResFrames-1 );
		
		pCurrSpectralFrameResidual = handleDB->GetSpectralFrame(synthState.GetAttackTimbre().GetPosition(),
																synthState.GetFrameCounterAttackResidual() );
		
		pSynthFrame.SetResidualPtr(&pCurrSpectralFrameResidual->GetResidualSpec()); //BUGFRAME? //SEGMENTBUG
		
		
		synthState.SetFrameCounterAttackResidual( synthState.GetFrameCounterAttackResidual() + 1 );
		
		// spectral interpolation process starts after 'mInterpolationstart' Frames
		if( synthState.GetFrameCounterBase() >= synthState.GetInterpolStart() )
			{
				synthState.SetIPFactor( mInterpolFactor * 
										( synthState.GetFrameCounterBase() - synthState.GetInterpolStart() )
										/ synthState.GetInterpolDuration() );
				if ( synthState.GetFrameCounterBase() >= synthState.GetInterpolEnd() )
					synthState.SetIPFactor( mInterpolFactor );
			}
		else
			{
				synthState.SetIPFactor( 0 );
			}

		mOut_InLoopSynthesis.SendControlAsBoolean( synthState.GetInLoopSynthesis() );
		mOutUseRandomDeviations.SendControlAsBoolean( mUseRandomDeviations );
		mOutUseRandomLoop.SendControlAsBoolean( mUseRandomLoop );
		mOutRandomRange.SendControl( mRandomRange );
		mOutTargetFreq.SendControl( synthState.GetCurrentStatTemplateFundFreq() );
		mOutMagInterpolFactor.SendControl( synthState.GetIPFactor() );
		mOutMagGain.SendControl( synthState.GetIndividualGain() );

	/*	mpInterpolPO.DoMagInterpolation( *mpCurrSpectralFrameBase,
										  *mpCurrPeakArrayTarget,
	//									  mpParams->GetCurrentStatTemplateFundFreq(),
	//									  mIPFactor,
	//									  mIndividualGain,
										  *pSynthFrame );
	*/
		double pitchCorrectionFactor=1.0;
		if  ( mUsePitchCorrection && pSynthFrame.GetFundFreq()!=0)
			pitchCorrectionFactor= synthState.GetPitchReference() /pSynthFrame.GetFundFreq();
		synthState.SetLastPitchCorrectionFactor( pitchCorrectionFactor ); // used in transition

		mOutPitchFactor.SendControl( synthState.GetPitchModFactor()*pitchCorrectionFactor );
	//	mpInterpolPO.DoPitchMod(*pSynthFrame);
	}

}
